﻿/* ============================================================
* Author:       xieqin
* Time:         2014/11/27 11:18:27
* FileName:     ModuleManager
* Purpose:      Module管理器
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Event;
using Log;
using ServerLib;

using System.Reflection;

namespace Module
{
    //ModuleManager实现
    public class ModuleManager : IModuleManager
    {
        protected HashSet<ILogicModule> setModules = new HashSet<ILogicModule>();

        public ModuleManager()
        {
        }

        public void RegisterModule(ILogicModule module)
        {
            setModules.Add(module);
        }

        public bool Init()
        {
            foreach (var module in setModules)
            {
                if(!module.Init())
                {
                    return false;
                }
            }

            foreach (var module in setModules)
            {
                if (!module.PostInit())
                {
                    return false;
                }
            }

            return true;
        }

        public void Destroy()
        {
            foreach (var module in setModules)
            {
                module.PreDestroy();
            }
            
            foreach (var module in setModules)
            {
                module.Destroy();
            }

            setModules.Clear();
        }

        public void Tick()
        {
            foreach (var module in setModules)
            {
                DateTime startTime = DateTime.Now;

                module.Tick();

                TimeSpan span = DateTime.Now - startTime;
                if (span.TotalMilliseconds > 50.0f)
                {
                    if (span.TotalMilliseconds > 500.0f)
                    {
                        LogSys.Warn($"[ModuleUpdate]delta = {span.TotalMilliseconds} module = {module.GetType().Name}");
                    }
                    else
                    {
                        LogSys.Info($"[ModuleUpdate]delta = {span.TotalMilliseconds} module = {module.GetType().Name}");
                    }
                }
            }
        }
    }
}
